/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.empfang.dekoder;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Meldung;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3MeldungsDaten;

/**
 * Klasse zum Zugriff auf Dateien nach TIC3-Info Spezifikation.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3InfoDatei.java 64239 2016-11-09 12:34:02Z gieseler $
 */
public class TIC3InfoDatei {

	/** TIC-Info Schema Location. */
	private final String ticSchemaDatei = "de/bsvrz/kex/lms/vew/TIC Info XML.xsd";

	/** die Datei. */
	private final File ticDatei;

	/**
	 * Konstruktor aus Datei.
	 *
	 * @param datei
	 *            Datei
	 */
	public TIC3InfoDatei(final File datei) {
		ticDatei = datei;
	}

	/**
	 * Erzeugt aus der Datei eine Liste von enthaltenen TIC3-Meldungen.
	 *
	 * @param mitVerifikation
	 *            legt fest, ob die empfangenen Dateien gegen das Schema
	 *            validiert werden sollen
	 * @return {@link TIC3InfoDateiDaten}
	 */
	public TIC3InfoDateiDaten bildeMeldungen(final boolean mitVerifikation) {
		final TIC3InfoDateiDaten daten = new TIC3InfoDateiDaten();

		final Document doc;
		try {
			doc = getDocument(mitVerifikation);
		} catch (final TIC3InfoDateiException e) {
			daten.setExeption(new TIC3InfoDateiException(e.getMessage()));
			return daten;
		}

		final NodeList nl = doc.getElementsByTagName(TIC3Meldung.XML_ELEMENT_MELDUNG);

		for (int i = 0; i < nl.getLength(); i++) {
			try {
				final TIC3MeldungsDaten meldung = new TIC3MeldungsDaten(nl.item(i));
				daten.addMeldung(meldung);
			} catch (final TIC3Exception | NumberFormatException e) {
				// speichere die erste Exception
				if (daten.getExeption() == null) {
					daten.setExeption(new TIC3InfoDateiException(e.getMessage()));
				}
			}
		}

		return daten;
	}

	/**
	 * Erzeugt das Document zur Datei. (Falls eine Verifikation der Datei gegen
	 * das Schema einmal m&ouml;glich ist, sollte dies hier erfolgen)
	 *
	 * @param mitVerifikation
	 *            legt fest, ob die empfangenen Dateien gegen das Schema
	 *            validiert werden sollen
	 * @return Document
	 * @throws TIC3InfoDateiException
	 *             bei Fehlern
	 */
	private Document getDocument(final boolean mitVerifikation) throws TIC3InfoDateiException {
		Document doc = null;

		final DocumentBuilderFactory docfactory = DocumentBuilderFactory.newInstance();

		if (mitVerifikation) {
			final SchemaFactory schemafactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
			final URL schemaurl;
			try {
				schemaurl = new URL("file://" + ticSchemaDatei);

				// final URL schemaurl =
				// ClassLoader.getSystemResource(_ticSchemaDatei);
				// Validierung gegen Schema nicht möglich
				// Aussage Herr Kuske, Fa. GEWI
				final Schema ticschema = schemafactory.newSchema(schemaurl);
				docfactory.setSchema(ticschema);
				docfactory.setValidating(true);
			} catch (final MalformedURLException e) {
				throw new TIC3InfoDateiException(e.getMessage());
			} catch (final SAXException e) {
				throw new TIC3InfoDateiException(e.getMessage());
			}
		}

		InputStream is = null;
		try {
			final DocumentBuilder parser = docfactory.newDocumentBuilder();
			is = new FileInputStream(ticDatei);
			doc = parser.parse(is);
		} catch (final Exception e) {
			throw new TIC3InfoDateiException(e.getMessage());
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
				}
			}
		}

		return doc;
	}
}
